"""
Core skeleton component
"""
import codecs
import collections
import datetime
import functools
import logging
import optparse
import os
import shutil
import sys
import weakref

from skeleton.utils import (
    get_loggger, get_file_mode, vars_to_optparser, prompt)


_LOG = get_loggger(__name__)


class SkeletonError(Exception):
    """Root exception"""


class TemplateKeyError(KeyError, SkeletonError):
    """Raised by Skeleton when a template required an unknown variable
    """

    #: Name of the unexpected variable
    variable_name = None

    #: Path to the file which cannot be formatted
    file_path = None

    def __init__(self, variable_name, file_path):
        super(TemplateKeyError, self).__init__(variable_name)
        self.variable_name = variable_name
        self.file_path = file_path

    def __str__(self):
        return ("Found unexpected variable %r in %r."
            % (self.variable_name, self.file_path,))


class FileNameKeyError(KeyError, SkeletonError):
    """Raised by Skeleton when a file name cannot be formatted
    """

    #: Name of the unexpected variable
    variable_name = None

    #: Path to the file which the name cannot be formatted
    file_path = None

    def __init__(self, variable_name, file_path):
        super(FileNameKeyError, self).__init__(variable_name)
        self.variable_name = variable_name
        self.file_path = file_path

    def __str__(self):
        return ("Found unexpected variable %r in file name %r"
            % (self.variable_name, self.file_path))


class ValidateError(SkeletonError):
    """Raised if a variable value is invalid .
    """


def run_requirements_last(skel_method):
    """Decorator for Skeleton methods

    The return wrapper will run the same method of the required
    skeleton instances after the wrapped method exists.
    """
    def wrapper(self, *args, **kw):
        """Method wrapper."""
        result = skel_method(self, *args, **kw)
        for skel in self.required_skeletons_instances:
            if hasattr(skel, skel_method.__name__):
                getattr(skel, skel_method.__name__)(*args, **kw)
        return result
    functools.update_wrapper(wrapper, skel_method)
    return wrapper


def run_requirements_first(skel_method):
    """Decorator for Skeleton methods

    The return wrapper will first run the same method of the required
    skeleton instances.
    """
    def wrapper(self, *args, **kw):
        """Method wrapper."""
        for skel in self.required_skeletons_instances:
            if hasattr(skel, skel_method.__name__):
                getattr(skel, skel_method.__name__)(*args, **kw)
        return skel_method(self, *args, **kw)
    functools.update_wrapper(wrapper, skel_method)
    return wrapper


class Skeleton(collections.MutableMapping):
    """Skeleton Class.

    It should have a `src` attribute set to the path to the skeleton folder
    (relative to the class module) and a list of variables, the `variables`
    attribute, the skeleton template files require. The variable should be an
    object with `name`, `display_name` and `full_description` attributes,
    and a prompt method which prompt the user for the variable value
    and return it. You can use `skeleton.Var`.

    By default a template file ends with "_tmpl" (see the `template_suffix`
    attribute), is UTF-8 encoded (`file_encoding` attribute) and will be
    formatted by Python 2.6+ string Formatter.

    You can set an alternative formatter by overwriting the
    `template_formatter` method. It takes for argument the template to parse
    and self for variable mapping.

    If the skeleton require other skeleton to be run first, list them in the
    required_skeletons attribute.
    """

    #: Path to skeleton folder, relative to Skeleton module
    src = None

    #: List of variable required by templates
    variables = []

    #: List of Skeleton to apply first
    required_skeletons = []

    #: Encoding of template file (UTF-8 by default)
    file_encoding = 'UTF-8'

    template_suffix = '_tmpl'
    run_dry = False

    def __init__(self, skeleton=None, **kw):
        self._required_skeletons_instances = None
        self._defaults = {}

        if isinstance(skeleton, Skeleton):
            self.set_variables = weakref.proxy(skeleton)
        else:
            self.set_variables = {'year': datetime.datetime.utcnow().year}
            if skeleton is not None:
                self.set_variables.update(skeleton)
        self.set_variables.update(kw)

        for var in self.variables:
            if var.default is not None:
                self._defaults[var.name] = var.default

    @property
    def required_skeletons_instances(self):
        """
        Return the skeletons required by this skeleton
        """
        if self._required_skeletons_instances is None:
            self._required_skeletons_instances = []
            for skel_class in self.required_skeletons:
                skel = skel_class(self)
                self._required_skeletons_instances.append(skel)
        return self._required_skeletons_instances

    @property
    def real_src(self):
        """
        Absolute Path to skeleton directory (read-only).
        """
        if self.src is None:
            raise AttributeError(
                "The src attribute of the %s Skeleton is not set" %
                self.__class__.__name__
                )

        mod = sys.modules[self.__class__.__module__]
        mod_dir = os.path.dirname(mod.__file__)
        skel_path = os.path.join(mod_dir, self.src)

        if not os.path.exists(skel_path):
            raise AttributeError("No skeleton at %r" % skel_path)
        return skel_path

    def __contains__(self, key):
        return key in self.set_variables or key in self._defaults

    def __delitem__(self, key):
        self.set_variables.__delitem__(key)

    def __getitem__(self, key):
        try:
            return self.set_variables[key]
        except KeyError:
            pass
        try:
            return self._defaults[key]
        except KeyError:
            pass
        raise KeyError("%s is not set and has no default value" % key)

    def __iter__(self):
        return iter(self.keys())

    def __len__(self):
        return len(self.set_variables) + len(self._defaults)

    def __setitem__(self, key, value):
        self.set_variables[key] = value

    def keys(self):
        """
        Return names of variables that are set or have a value
        """
        return list(set(self.set_variables.keys() + self._defaults.keys()))

    def update(self, *args, **kw):
        """
        Update the set_variables attribute
        """
        self.set_variables.update(*args, **kw)

    @run_requirements_last
    def check_variables(self):
        """
        Raise a KeyError if any required variable is missing.
        """
        for var in self.variables:
            self.__getitem__(var.name)

    @run_requirements_last
    def get_missing_variables(self):
        """
        Prompt user for any missing variable
        (even the ones with a default value).
        """
        for var in self.variables:
            if var.name not in self.set_variables:
                self[var.name] = var.do_prompt()
            else:
                _LOG.debug("Variable %r already set", var.name)

    @run_requirements_first
    def write(self, dst_dir, run_dry=False):
        """Apply skeleton to `dst_dir`.

        Copy files and folders from the `src` folder to the `dst_dir`.
        If `dst_dir` doesn't exist, it will be created.

        The file name are formatted by the template formatter so that file
        names can do dynamically generated. Make sure that any special
        characters for the formatters are escaped.

        If the file name ends by "_tmpl" its content will be formatted by the
        template formatter.

        Raises:

        - `KeyError` if a variable is missing and doesn't have a default.
        - `TemplateKeyError` if it found an unexpected variable in a template.
        - `FileNameKeyError` if it found an unexpected variable in a file name.
        - IOError if it cannot read the skeleton files, or cannot create
          files and folder.
        """
        self.run_dry = run_dry

        _LOG.info(
            "Rendering %s skeleton at %r...",
            self.__class__.__name__,
            dst_dir)

        self.check_variables()

        if not os.path.exists(dst_dir):
            self._mkdir(dst_dir)

        real_src = self.real_src
        real_src_len = len(real_src)
        _LOG.debug("Getting skeleton from %r" % real_src)

        for dir_path, dir_names, file_names in os.walk(real_src):
            rel_dir_path = dir_path[real_src_len:].lstrip(r'\/')

            #copy files
            for file_name in file_names:
                src = os.path.join(dir_path, file_name)
                dst = os.path.join(
                    dst_dir,
                    rel_dir_path,
                    self._format_file_name(file_name, dir_path)
                    )
                self._copy_file(src, dst)

            #copy directories
            for dir_name in dir_names:
                src = os.path.join(dir_path, dir_name)
                dst = os.path.join(
                    dst_dir,
                    rel_dir_path,
                    self.template_formatter(dir_name))
                self._mkdir(dst, like=src)

    def run(self, dst_dir, run_dry=False):
        """Like write() but prompt user for missing variables.

        Raises:

        - `TemplateKeyError` if it found an unexpected variable in a template.
        - `FileNameKeyError` if it found an unexpected variable in a file name.
        - IOError if it cannot read the skeleton files, or cannot create
          files and folder.
        """
        self.get_missing_variables()
        self.write(dst_dir, run_dry=run_dry)

    @classmethod
    def cmd(cls, argv=None, **kw):
        """
        Convenient method to set a logger, an optpaser and run the skeleton
        """

        skel = cls(**kw)

        parser = skel.configure_parser()
        options, args = parser.parse_args(argv)
        if len(args) != 1:
            parser.error("incorrect number of arguments")

        logging.basicConfig(
            level=options.verbose_,
            format="%(levelname)s - %(message)s"
            )

        for var in skel.variables:
            value = getattr(options, var.name)
            if value is not None:
                skel[var.name] = value

        skel.run(args[0])

    def configure_parser(self):
        """Configure parser for Skeleton.cmd().
        """
        parser = optparse.OptionParser(usage="%prog [options] dst_dir")
        parser.add_option("-q", "--quiet",
            action="store_const", const=logging.FATAL, dest="verbose_")
        parser.add_option("-v", "--verbose",
            action="store_const", const=logging.INFO, dest="verbose_")
        parser.add_option("-d", "--debug",
            action="store_const", const=logging.DEBUG, dest="verbose_")
        parser.set_default('verbose_', logging.ERROR)

        parser = vars_to_optparser(self.variables, parser=parser)
        return parser

    def template_formatter(self, template):
        """Return a formatted version of of the string `template`.

        Raises a KeyError if a variable is missing.
        """
        return template.format(**self)

    def _format_file_name(self, file_name, dir_path):
        try:
            return self.template_formatter(file_name)
        except (KeyError,), exc:
            raise FileNameKeyError(
                exc.args[0],
                os.path.join(dir_path, file_name)
                )

    def _mkdir(self, path, like=None):
        """Create a directory (using os.mkdir)

        Only log the event if self.run_dry is True.
        """
        _LOG.info("Create directory %r", path)
        if not self.run_dry and not os.path.exists(path):
            os.mkdir(path)
        if like is not None:
            self._set_mode(path, like)

    def _copy_file(self, src, dst):
        """Copy src file to dst and format dst if src is a template.

        The template suffix should be removed from dst.
        """
        if dst.endswith(self.template_suffix):
            try:
                self._format_file(src, dst[:-len(self.template_suffix)])
            except (KeyError,), exc:
                raise TemplateKeyError(exc.args[0], src)
        else:
            self._copy_static_file(src, dst)

    def _copy_static_file(self, src, dst):
        """Copy file and mode.

        Only log the event if self.run_dry is True.
        """
        _LOG.info("Copy %r to %r", src, dst)
        if not self.run_dry:
            shutil.copyfile(src, dst)
        self._set_mode(dst, like=src)

    def _format_file(self, src, dst):
        """Copy src to dst and format it.

        Raises a KeyError if a variable is missing.
        """
        _LOG.info("Creating %r from %r template...", dst, src)
        if not self.run_dry:
            fd_src = None
            fd_dst = None
            try:
                fd_src = codecs.open(src, encoding=self.file_encoding)
                fd_dst = codecs.open(dst, 'w', encoding=self.file_encoding)
                fd_dst.write(self.template_formatter(fd_src.read()))
            finally:
                if fd_src is not None:
                    fd_src.close()
                if fd_dst is not None:
                    fd_dst.close()
        self._set_mode(dst, like=src)

    def _set_mode(self, path, like):
        """
        Set mode of `path` with the mode of `like`.
        """
        _LOG.info("Set mode of %r to '%o'", path, get_file_mode(like))
        if not self.run_dry:
            shutil.copymode(like, path)


class Var(object):
    """Define a template variable.

    The variable names should follow pep8 guidelines about variable names.
    pep8 variable are easier to set with a Skeleton constructor and you should
    not assume the skeleton template formatter can use any name formatting.
    """
    _prompt = staticmethod(prompt)

    def __init__(self, name, description=None, default=None, intro=None):
        self.name = name
        self.description = description
        self.default = default
        self.intro = intro

    def __repr__(self):
        return u'<%s %s default=%r>' % (
            self.__class__.__name__, self.name, self.default,)

    @property
    def display_name(self):
        """Return a titled version of name were "_" are replace by spaces.

        Allows to get nice looking name at prompt while following pip8 guidance
        (a Var name can be use as argument of skeleton to set the variable).
        """
        return self.name.replace('_', ' ').title()

    @property
    def full_description(self):
        """Return the name of the variable and a description if description
        is set.
        """
        if self.description:
            return u'%s (%s)' % (self.display_name, self.description,)
        else:
            return self.display_name

    @property
    def prompt(self):
        """Message to Prompt"""
        prompt_ = u'Enter %s' % self.full_description
        if self.default is not None:
            prompt_ += u' [%r]' % self.default
        prompt_ += u': '
        return prompt_

    def do_prompt(self):
        """Prompt user for variable value and return the validated value

        It will keep prompting the user until it receive a valid value.
        By default, a value is valid if it is not a empty string string or if 
        the variable has a default.

        If the user value is empty and the variable has a default, the default
        value is returned.

        """
        if self.intro is not None:
            print self.intro

        prompt_ = self.prompt
        while True:
            try:
                return self.validate(self._prompt(prompt_))
            except (ValidateError,), exc:
                print str(exc)


    def validate(self, response):
        """Checks the user has given a non empty value or that the variable has
        a default.

        Returns the valide value or the default.

        Raises a ValidateError if the response is invalid.
        """
        if response:
            return response
        elif self.default is not None:
            return self.default
        else:
            raise ValidateError("%s is required" % self.display_name)


class Bool(Var):
    """Var accepting "y/n"

    """

    @property
    def full_description(self):
        """Return the name of the variable, a description if description
        is set, and the y/N choice.

        """
        if self.description:
            return u'%s (%s - y/N)' % (self.display_name, self.description,)
        else:
            return u'%s (y/N)' % self.display_name

    @property
    def prompt(self):
        """Message to Prompt (with default converted to "y" or "n")

        """
        prompt_ = u'Enter %s' % self.full_description
        if self.default is not None:
            prompt_ += u' [%r]' % (self.default and 'y' or 'N')
        prompt_ += u': '
        return prompt_

    def validate(self, response):
        """Checks the response is either Y, YES, N or NO, or that the variable
        has a default value.

        Raises a ValidateError exception if the response wasn't recognized or 
        if no value was given and one is required.

        """
        response = response.strip().upper()
        if response in ('Y', 'YES',):
            return True
        elif response in ('N', 'NO',):
            return False
        elif response == '':
            if self.default is not None:
                return self.default
            else:
                raise ValidateError("%s is required" % self.display_name)
        else:
            raise ValidateError('enter either "Y" for yes or "N" or no')



"""
Test tools
"""
import unittest
import tempfile
import shutil

from skeleton import Var
import os


class Mock(object):
    """Basic mock object"""

    def __init__(self):
        self.return_value = None
        self.side_effect = None
        self.call_args_list = []

    def __call__(self, *args, **kw):
        self.call_args_list.append((args, kw,))

        if self.side_effect is not None:
            return self.side_effect(*args, **kw)
        elif self.return_value is not None:
            return self.return_value
        else:
            raise Exception("No return value or side effect set.")

    @property
    def call_count(self):
        """
        Return number of time the mock object has been called
        """
        return len(self.call_args_list)

    @property
    def called(self):
        """Check if the mock object has been called"""
        return self.call_count > 0


class TempDir(object):
    """
    Wrapper class around tempfile.mkdtemp compatible with the "with statement"
    """

    def __init__(self):
        self.path = None

    def create(self):
        """
        Create temporary directory.

        set the path attribute to the this directory path

        """
        self.path = tempfile.mkdtemp()
        return self

    def remove(self):
        """
        remove temporary directory.
        """
        shutil.rmtree(self.path)

    def join(self, *args):
        return os.path.join(self.path, *args)

    def exists(self, *args):
        return os.path.exists(self.join(*args))

    __enter__ = create

    def __exit__(self, exc_type, value, traceback):
        self.remove()


class TestCase(unittest.TestCase):
    """
    Basic test case.
    """

    def setUp(self):
        """
        Mock Var._prompt
        """
        super(TestCase, self).setUp()
        self.input_mock = Mock()
        self._input = Var._prompt
        Var._prompt = self.input_mock

    def tearDown(self):
        """
        Reset Var._prompt and remove the temporary directory.
        """
        super(TestCase, self).tearDown()
        Var._prompt = self._input



"""
Tests the BDS, GPL and LGPL skeleton
"""
import subprocess
import sys
import unittest

from skeleton.examples.basicpackage import BasicPackage, main
from skeleton.examples.licenses import (
    BSD, BSD_THIRD_CLAUSE, GPL, LGPL, NoLicense, LicenseChoice,
    )
from skeleton.examples.mkmodule import BasicModule
from skeleton.tests.utils import TestCase, TempDir


class TestBSD(TestCase):
    """Tests skeleton.example.license.BSD."""

    def test_write_2clause(self):
        """Tests write of a 2-clauses BSD license."""
        skel = BSD(author='Damien Lebrun', organization='')

        with TempDir() as tmp:
            skel.write(tmp.path)

            self.assertEqual(skel['third_clause'], '')
            self.assertTrue(tmp.exists('LICENSE'))

    def test_write_3clause(self):
        """Tests write of a 3-clauses BSD license."""
        skel = BSD(author='Damien Lebrun', organization='Foo inc')

        with TempDir() as tmp:
            skel.write(tmp.path)

            self.assertEqual(
                skel['third_clause'],
                BSD_THIRD_CLAUSE.format(organization='Foo inc')
                )
            self.assertTrue(tmp.exists('LICENSE'))


class TestGPL(TestCase):
    """Tests skeleton.example.license.GPL."""

    def test_write(self):
        """Tests write of a GPL skeleton"""
        skel = GPL(author='Damien Lebrun', project_name='Foo')

        with TempDir() as tmp:
            skel.write(tmp.path)

            self.assertTrue(tmp.exists('LICENSE'))
            self.assertTrue(tmp.exists('COPYING'))


class TestLGPL(TestCase):
    """Tests skeleton.example.license.LGPL."""

    def test_write(self):
        """Tests write of a LGPL skeleton"""
        skel = LGPL(author='Damien Lebrun', project_name='Foo')

        with TempDir() as tmp:
            skel.write(tmp.path)

            self.assertTrue(tmp.exists('LICENSE'))
            self.assertTrue(tmp.exists('COPYING'))
            self.assertTrue(tmp.exists('COPYING.LESSER'))


class TestNoLicense(TestCase):
    """Tests the NoLicense skeleton."""

    def test_write(self):
        """Tests write of a NoLicense skeleton"""
        skel = NoLicense(author='Damien Lebrun')

        with TempDir() as tmp:
            skel.write(tmp.path)

            self.assertTrue(tmp.exists('LICENSE'))


class TestLicenseChoice(TestCase):
    """Tests the LicenseChoice skeleton."""

    def test_licence_skel_default(self):
        """Tests the default license_ske property"""
        skel = LicenseChoice(author='Damien Lebrun', project_name='Foo')

        self.assertTrue(isinstance(skel.license_skel, NoLicense))
        self.assertEqual(skel.license_skel['author'], 'Damien Lebrun')

    def test_bsd_licence_skel(self):
        """Tests for a BSD license_ske property"""
        skel = LicenseChoice(
            author='Damien Lebrun', project_name='Foo', license='BSD'
            )

        self.assertTrue(isinstance(skel.license_skel, BSD))

    def test_gpl_licence_skel(self):
        """Tests for a GPL license_ske property"""
        skel = LicenseChoice(
            author='Damien Lebrun', project_name='Foo', license='GPL'
            )
        self.assertTrue(isinstance(skel.license_skel, GPL))

    def test_lgpl_licence_skel(self):
        """Tests for a LGPL license_ske property"""
        skel = LicenseChoice(
            author='Damien Lebrun', project_name='Foo', license='LGPL'
            )

        self.assertTrue(isinstance(skel.license_skel, LGPL))

    def test_lgpl_run(self):
        """Tests run of a LicenceChoice with license set to "LGPL"
        """
        resps = ['Foo', 'Damien Lebrun', 'dinoboff@gmail.com', 'LGPL', ]
        self.input_mock.side_effect = lambda x: resps.pop(0)

        skel = LicenseChoice()
        with TempDir() as tmp:
            skel.run(tmp.path)

            self.assertTrue(tmp.exists('LICENSE'))
            self.assertTrue(tmp.exists('COPYING'))
            self.assertTrue(tmp.exists('COPYING.LESSER'))

    def test_lgpl_write(self):
        """Tests write of a LicenceChoice with license set to "LGPL"
        """
        skel = LicenseChoice(
            author='Damien Lebrun', project_name='Foo', license='LGPL'
            )
        with TempDir() as tmp:
            skel.write(tmp.path)

            self.assertTrue(tmp.exists('LICENSE'))
            self.assertTrue(tmp.exists('COPYING'))
            self.assertTrue(tmp.exists('COPYING.LESSER'))

    def test_lgpl_write_fails(self):
        """Tests write of a LicenceChoice fails if a key is missing
        """
        skel = LicenseChoice(author='Damien Lebrun', license='LGPL')

        with TempDir() as tmp:
            self.assertRaises(KeyError, skel.write, tmp.path)


class TestBasicModule(TestCase):
    """Tests BasicModule Skeleton
    """

    def test_write(self):
        """Tests BasicModule.write()
        """
        skel = BasicModule(
            module_name='foo',
            author='Damien Lebrun',
            author_email='dinoboff@gmail.com',
            )

        with TempDir() as tmp:
            skel.write(tmp.path)

            self.assertTrue(tmp.exists('README.rst'))
            self.assertTrue(tmp.exists('setup.py'))
            self.assertTrue(tmp.exists('foo.py'))


class TestBasicPackage(TestCase):
    """Tests the BasicPackage Skeleton
    """

    def test_write(self):
        """Tests skeleton.examples.basicpackage.BasicPackage with a single package
        """
        skel = BasicPackage(
            project_name='foo',
            package_name='foo',
            author='Damien Lebrun',
            author_email='dinoboff@gmail.com'
            )

        with TempDir() as tmp:
            skel.write(tmp.path)

            self.assertEqual(skel['ns_packages'], [])
            self.assertEqual(skel['packages'], ['foo'])

            self.assertTrue(tmp.exists('distribute_setup.py'))
            self.assertTrue(tmp.exists('MANIFEST.in'))
            self.assertTrue(tmp.exists('README.rst'))
            self.assertTrue(tmp.exists('LICENSE'))
            self.assertTrue(tmp.exists('setup.py'))
            self.assertTrue(tmp.exists('foo/__init__.py'))

    def test_write_with_bsd(self):
        """Tests skeleton.examples.basicpackage.BasicPackage add BSD license
        """
        skel = BasicPackage(
            project_name='foo',
            package_name='foo',
            author='Damien Lebrun',
            author_email='dinoboff@gmail.com',
            license='BSD'
            )

        with TempDir() as tmp:
            skel.write(tmp.path)
            self.assertTrue(tmp.exists('LICENSE'))

            fragment = """
            Redistributions of source code must retain the above copyright notice
            """.strip()
            with open(tmp.join('LICENSE')) as license_file:
                content = license_file.read()
                self.assertTrue(fragment in content)

    def test_write_namespaces(self):
        """Tests skeleton.examples.basicpackage.BasicPackage with namespaces
        """
        skel = BasicPackage(
            project_name='foo.bar.baz',
            package_name='foo.bar.baz',
            author='Damien Lebrun',
            author_email='dinoboff@gmail.com'
            )

        with TempDir() as tmp:
            skel.write(tmp.path)

            self.assertEqual(set(skel['ns_packages']), set(['foo', 'foo.bar']))
            self.assertEqual(
                set(skel['packages']),
                set(['foo', 'foo.bar', 'foo.bar.baz']))

            self.assertTrue(tmp.exists('distribute_setup.py'))
            self.assertTrue(tmp.exists('MANIFEST.in'))
            self.assertTrue(tmp.exists('README.rst'))
            self.assertTrue(tmp.exists('LICENSE'))
            self.assertTrue(tmp.exists('setup.py'))
            self.assertTrue(tmp.exists('foo/__init__.py'))
            self.assertTrue(tmp.exists('foo/bar/__init__.py'))
            self.assertTrue(tmp.exists('foo/bar/baz/__init__.py'))

    def test_main(self):
        """Tests basicpackage.main()"""
        resps = ['foo', 'foo', 'Damien Lebrun', 'dinoboff@gmail.com', 'BSD', '']
        self.input_mock.side_effect = lambda x: resps.pop(0)

        with TempDir() as tmp:
            main([tmp.path])

            self.assertTrue(tmp.exists('distribute_setup.py'))
            self.assertTrue(tmp.exists('MANIFEST.in'))
            self.assertTrue(tmp.exists('README.rst'))
            self.assertTrue(tmp.exists('foo/__init__.py'))

            setup = 'setup.py'
            # Test egg_info can be run
            proc = subprocess.Popen(
                [sys.executable, setup, 'egg_info'],
                shell=False,
                stdout=subprocess.PIPE,
                cwd=tmp.path)
            self.assertEqual(proc.wait(), 0)

            # Test classifiers
            proc = subprocess.Popen(
                [sys.executable, setup, '--classifiers'],
                shell=False,
                stdout=subprocess.PIPE,
                cwd=tmp.path)
            self.assertEqual(proc.wait(), 0)
            classifiers = proc.stdout.read().decode().splitlines()
            self.assertTrue(
                "License :: OSI Approved" in classifiers)
            self.assertTrue(
                "License :: OSI Approved :: BSD License" in classifiers)


def suite():
    """Get all licence releated test"""
    tests = unittest.TestSuite()
    tests.addTest(unittest.TestLoader().loadTestsFromTestCase(TestBSD))
    tests.addTest(unittest.TestLoader().loadTestsFromTestCase(TestGPL))
    tests.addTest(unittest.TestLoader().loadTestsFromTestCase(TestLGPL))
    tests.addTest(unittest.TestLoader().loadTestsFromTestCase(TestNoLicense))
    tests.addTest(unittest.TestLoader().loadTestsFromTestCase(TestBasicModule))
    tests.addTest(unittest.TestLoader().loadTestsFromTestCase(TestBasicPackage))
    return tests

if __name__ == "__main__":
    unittest.main()



"""
Tests in skeleton.utils.*
"""
import optparse
import unittest

from skeleton.core import Var
from skeleton.tests.utils import TestCase, TempDir
from skeleton.utils import insert_into_file, vars_to_optparser


class TestInsertIntoFile(TestCase):
    """Tests for in skeleton.utils.insert_into_file"""

    def test_insert_into_file(self):
        """Tests skeleton.insert_into_file()"""
        with TempDir() as tmp_dir:
            target = tmp_dir.join('test.txt')
            with open(target, 'w') as f_target:
                f_target.write("""foo\n# -*- insert here -*- #\nbaz\n""")

            insert_into_file(target, 'insert here', 'bar\n')
            with open(target) as f_target:
                self.assertEqual(
                    f_target.readlines(),
                    ['foo\n', '# -*- insert here -*- #\n', 'bar\n', 'baz\n']
                    )

    def test_insert_with_indent(self):
        """Tests skeleton.insert_into_file() with indent to keep"""
        with TempDir() as tmp_dir:
            target = tmp_dir.join('test.txt')
            with open(target, 'w') as f_target:
                f_target.write("""foo\n  # -*- insert here -*- #\nbaz\n""")

            insert_into_file(target, 'insert here', 'bar\nfooz\n')
            with open(target) as f_target:
                self.assertEqual(
                    f_target.read().strip(),
                    'foo\n  # -*- insert here -*- #\n  bar\n  fooz\nbaz'
                    )

    def test_insert_with_indent_to_lose(self):
        """Tests skeleton.insert_into_file() with indent to lose"""
        with TempDir() as tmp_dir:
            target = tmp_dir.join('test.txt')
            with open(target, 'w') as f_target:
                f_target.write("""foo\n  # -*- insert here -*- #\nbaz\n""")

            insert_into_file(target, 'insert here', 'bar\n', keep_indent=False)
            with open(target) as f_target:
                self.assertEqual(
                    f_target.readlines(),
                    ['foo\n', '  # -*- insert here -*- #\n', 'bar\n', 'baz\n']
                    )

    def test_insert_and_remove_marker(self):
        """Tests skeleton.insert_into_file() with keep_marker off"""
        with TempDir() as tmp_dir:
            target = tmp_dir.join('test.txt')
            with open(target, 'w') as f_target:
                f_target.write("""foo\n# -*- insert here -*- #\nbaz\n""")

            insert_into_file(target, 'insert here', 'bar\n', keep_marker=False)
            with open(target) as f_target:
                self.assertEqual(
                    f_target.readlines(),
                    ['foo\n', 'bar\n', 'baz\n']
                    )


class TestVarsToOptparser(unittest.TestCase):
    """Tests skeleton.utils.vars_to_optparser
    """

    def test_create_parser(self):
        """Tests vars_to_optparser create the OptParser if none are given
        """

        self.assertTrue(
            isinstance(vars_to_optparser([]), optparse.OptionParser)
            )

    def test_augment_parser(self):
        """Tests vars_to_optparser augment the parser given as argument
        """
        parser = optparse.OptionParser()
        self.assertTrue(vars_to_optparser([], parser) is parser)

    def test_pep8_var_name(self):
        """Tests long string formatting of options added by vars_to_optparser.

        All lower case with hyphen instead of underscore

        """
        parser = vars_to_optparser([Var('Foo'), Var('foo_bar')])
        self.assertEqual(parser.get_option('--foo').dest, 'Foo')
        self.assertEqual(parser.get_option('--foo-bar').dest, 'foo_bar')

    def test_default_metavar(self):
        """Tests option's metavar"""
        parser = vars_to_optparser([Var('Foo'), Var('bar_name')])
        self.assertEqual(parser.get_option('--foo').metavar, 'FOO')
        self.assertEqual(parser.get_option('--bar-name').metavar, 'NAME')


def suite():
    """Return tests for skeleton.utils.*  """
    tests = unittest.TestSuite()
    tests.addTest(
        unittest.TestLoader().loadTestsFromTestCase(TestInsertIntoFile))
    tests.addTest(
        unittest.TestLoader().loadTestsFromTestCase(TestVarsToOptparser))
    return tests


if __name__ == "__main__":
    unittest.main()



"""
Skeleton to create a basic package and the virtualenwrapper.project extension
to add it a template.
"""

from __future__ import with_statement

import logging
import os

from skeleton import Skeleton, Var
from skeleton.utils import get_loggger, insert_into_file
from skeleton.examples.licenses import LicenseChoice


_LOG = get_loggger(__name__)

NS_HEADER = """
__import__('pkg_resources').declare_namespace(__name__)
"""


class BasicPackage(Skeleton):
    """Create a new package package (with namespace support) with the setup.py,
    README.rst and MANIFEST.in files already setup.

    Require the following variables:

    - project_name;
    - package_name;
    - author;
    - and author_email.

    Todo: Allow to set a global defaults for author and author_email.
    Todo: Better README.rst content like Having a basic Installation and
    requirement section.
    Todo: Setup a test package and the distribute use_2to3 option.
    """

    src = 'basic-package'
    variables = [
        Var('project_name'),
        Var('package_name'),
        Var('author'),
        Var('author_email'),
        ]
    required_skeletons = [
        LicenseChoice,
        ]
    licence_classifiers = {
            'BSD': 'License :: OSI Approved :: BSD License',
            'GPL': ('License :: OSI Approved :: '
                        'GNU General Public License (GPL)'),
            'LGPL': ('License :: OSI Approved :: '
                        'GNU Library or Lesser General Public License (LGPL)'),
        }

    def write(self, dst_dir, run_dry=False):
        """Create package(s) dynamically.

        Overwrite the write method to add the NSPackages and Packages entry
        before skeleton write, and create the list of package they hold after
        the skeleton write.

        """
        self._set_packages_and_namespaces()
        super(BasicPackage, self).write(dst_dir, run_dry=run_dry)
        self._create_packages(dst_dir)
        self._add_classifier(dst_dir)

    def _set_packages_and_namespaces(self):
        """
        Create a list of package and namespaces suitable for setuptools.
        """
        self['ns_packages'] = []
        self['packages'] = [self['package_name']]

        # Add parent package to the list of package and namespaces
        current_ns = []
        package_parts = self['package_name'].split('.')
        for part in package_parts[:-1]:
            current_ns.append(part)
            parent_package = '.'.join(current_ns)
            self['ns_packages'].append(parent_package)
            self['packages'].append(parent_package)

    def _create_packages(self, dst_dir):
        """
        Create a packages listed in self['Packages']
        """
        packages = self.get('packages', [])
        packages.sort()
        for package in packages:
            init_body = ''
            if package in self.get('ns_packages', []):
                init_body = NS_HEADER
            self._create_package(dst_dir, package, init_body)

    def _create_package(self, dst_dir, package, init_body=''):
        """Create a package - directory and __init__.py file.

        The parent package should already exist.

        """
        package_part = package.split('.')
        path = os.path.join(dst_dir, *package_part)
        _LOG.info("Creating package %s" % package)
        if self.run_dry:
            return
        os.mkdir(path)
        with open(os.path.join(path, '__init__.py'), 'w') as init_file:
            init_file.write(init_body)

    def _add_classifier(self, dst_dir):
        """Add license classifiers

        """
        setup = os.path.join(dst_dir, 'setup.py')
        license_name = self.get('license', '').upper()
        if license_name not in self.licence_classifiers:
            return
        classifiers = [
            "License :: OSI Approved",
            self.licence_classifiers[license_name],
            ]

        insert_into_file(
            setup,
            "Classifiers",
            '\n'.join(['%r,' % cla for cla in classifiers])
            )


def virtualenv_warpper_hook(*args, **kw):
    """
    Create a new package package (with namespace support)
    with the setup.py, README.rst and MANIFEST.in files already setup.
    """
    logging.basicConfig(
        level=logging.INFO,
        format="%(levelname)s - %(message)s"
        )
    BasicPackage().run('src/')


def main(argv=None):
    """Bootstrap BasicPackage."""
    BasicPackage.cmd(argv)

if __name__ == '__main__':
    main()



"""
Skeleton Helpers
"""

from __future__ import with_statement
from contextlib import closing
import codecs
import logging
import optparse
import os
import re
import stat
import sys


VALID_OPTION_NAME = re.compile("[a-z]([\w\d]*[a-z0-9])?", re.IGNORECASE)


def get_loggger(name):
    """Get the logger for the given name and assign an dummy handler to it.
    """
    logger = logging.getLogger(name)
    logger.addHandler(NullHandler())
    return logger


def vars_to_optparser(variables, parser=None):
    """Augments the parser with option to set value for the list of variables.
    """
    if parser is None:
        parser = optparse.OptionParser()

    for var in variables:
        if not VALID_OPTION_NAME.match(var.name):
            continue
        parser.add_option(
            "--%s" % var.name.lower().replace('_', '-'),
            dest=var.name,
            help=var.full_description,
            metavar=var.name.split('_')[-1].upper())
    return parser


def get_file_mode(path):
    """
    Return the mode of a file, the part usable with os.chmod
    """
    return stat.S_IMODE(os.stat(path)[stat.ST_MODE])


def insert_into_file(
    file_path, marker, text,
    marker_tag="-*-", keep_indent=True, keep_marker=True, encoding="UTF-8"):
    """Insert text into file at specific markers.

    eg, for a file "test.txt" with::

        foo
        # -*- Insert Here -*-
        baz

    `-*- Insert Here -*-` is the marker; anything can be added in front or
    after. `insert_into_file('test.txt', 'Insert Here', 'bar')` would result
    with::

        foo
        # -*- Insert Here -*-
        bar
        baz

    Arguments:

    - file_path:  file to insert content into.
    - marker:  Marker to look for in the file.
    - text (unicode):  text to insert in the file.
    - marker_tag:  text surrounding the marker.
    - keep_indent: Should it insert the text with the same marker indent.
    - keep_marker: Should the marker be removed.
    - encoding: file encoding.
    """
    marker_pattern = re.escape('%s %s %s' % (marker_tag, marker, marker_tag,))
    marker_re = re.compile(r"^(\s*).*%s.*$" % marker_pattern)
    edited = False
    new_content = []
    with closing(
        codecs.open(file_path, 'r', encoding=encoding)) as opened_file:
        for line in opened_file:
            match = marker_re.match(line)
            if match is None:
                new_content.append(line.rstrip('\n\r'))
                continue

            edited = True

            if keep_marker:
                new_content.append(line.rstrip('\n\r'))

            if keep_indent:
                indent = match.groups()[0]
                for text_line in text.splitlines():
                    new_content.append('%s%s' % (indent, text_line,))
            else:
                for text_line in text.splitlines():
                    new_content.append(text_line)

    if not edited:
        return

    with closing(
        codecs.open(file_path, 'w', encoding=encoding)) as opened_file:
        for line in new_content:
            opened_file.write('%s%s' % (line, os.linesep,))


class NullHandler(logging.Handler):
    """Dummy log handler.
    """

    def emit(self, record):
        """
        Doesn't do anything with record
        """


def prompt(prompt_):
    """Wrapper around the raw_input builtin function

    Will return unicode in Python 2 like in Python 3
    """
    result = raw_input(prompt_)
    try:
        return result
        #return result.decode(sys.stdin.encoding)
    except AttributeError:
        return result



#!/usr/bin/env python
"""
Basic script to create an empty python package containing one module
"""
from skeleton import Skeleton, Var


class BasicModule(Skeleton):
    """
    Create an empty module with its etup script and a README file.
    """
    src = 'basic-module'
    variables = [
        Var('module_name'),
        Var('author'),
        Var('author_email'),
        ]


def main():
    """Basic command line bootstrap for the BasicModule Skeleton"""
    BasicModule.cmd()

if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Bootstrap installation of Distribute
import distribute_setup
distribute_setup.use_setuptools()

import sys

from setuptools import setup


def read_file(name):
    """
    Read file content
    """
    f = open(name)
    try:
        return f.read()
    except IOError:
        print("could not read %r" % name)
        f.close()


PROJECT = 'skeleton_stu'
VERSION = '0.6.2'
URL = 'http://dinoboff.github.com/skeleton'
AUTHOR = 'Damien Lebrun'
AUTHOR_EMAIL = 'dinoboff@gmail.com'
DESC = "Basic Template system for project skeleton."
LONG_DESC = read_file('README.rst') + '\n\n' + read_file('HISTORY.rst')

EXTRAS = {}

if sys.version_info > (3,):
    EXTRAS['use_2to3'] = True

setup(
    name=PROJECT,
    version=VERSION,
    description=DESC,
    long_description=LONG_DESC,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    license='BSD',
    packages=['skeleton', 'skeleton.tests', 'skeleton.examples'],
    test_suite='skeleton.tests',
    include_package_data=True,
    zip_safe=False,
    install_requires=[],
    extras_require={
        'virtualenv-templates':  [
            'virtualenvwrapper>=2.1.1',
            'virtualenvwrapper.project>=1.0'
            ],
    },
    entry_points={
        'virtualenvwrapper.project.template': [
            'package = skeleton.examples.basicpackage:virtualenv_warpper_hook',
            ],
       'console_scripts': [
            'skeleton_module = skeleton.examples.mkmodule:main',
            'skeleton_package = skeleton.examples.basicpackage:main',
            'skeleton_package_sphinx = skeleton.examples.sphinxpackage:main',
            ],
        },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.1',
        ],
    **EXTRAS
)



"""
Skeleton to create a project with sphinx support.
"""

from __future__ import with_statement

import logging
import os

from skeleton import Skeleton, Var
from skeleton.examples.basicpackage import BasicPackage
from skeleton.utils import get_loggger

_LOG = get_loggger(__name__)


class SphinxPackage(Skeleton):
    """Create a new package package with sphinx setup.
    includes features from basicpackage:
    Create a new package package (with namespace support) with the setup.py,
    README.rst and MANIFEST.in files already setup.

    Require the following variables:

    - project_name;
    - package_name;
    - author;
    - author_email;
    - license;
    - and short_description.


    """

    src = 'sphinx-package'
    variables = [
        Var('project_name'),
        Var('package_name'),
        Var('author'),
        Var('author_email'),
        Var('license'),
        Var('short_description')
        ]
    required_skeletons = [
        BasicPackage,
        ]


def main(argv=None):
    """Bootstrap SphinxPackage."""
    SphinxPackage.cmd(argv)

if __name__ == '__main__':
    main()



"""
Skeletons for BSD, GPL and LGPL licenses.
"""

from skeleton import Skeleton, Var


BSD_THIRD_CLAUSE = """
    - Neither the name of the {organization} nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.
"""


class NoLicense(Skeleton):
    """Had the a license with copyright notice only.

    """
    src = 'licenses/no-license'
    variables = [
        Var('author'),
        ]


class BSD(Skeleton):
    """Adds a 2 or 3 clauses BSD License.

    Requires author and organization variables.

    Sets a ThirdClause variable.

    """

    src = 'licenses/bsd'
    variables = [
        Var('author'),
        Var('organization',
            default='',
            description="required for a 3 clauses-BSD license - "
                "leave it empty for a 2 clauses-BSD license."
            ),
        ]

    def write(self, dst_dir, run_dry=False):
        """Set the ThirdClause if an organization name has been given.

        """
        if self.get('organization'):
            self['third_clause'] = self.template_formatter(BSD_THIRD_CLAUSE)
        else:
            self['third_clause'] = ''
        super(BSD, self).write(dst_dir, run_dry=run_dry)


class GPL(Skeleton):
    """
    Adds a GPL notice and the GPLv3 license.

    Requires Author and ProjectName variables.

    """

    src = 'licenses/gpl'
    variables = [
        Var('author'),
        Var('project_name'),
        ]


class LGPL(GPL):
    """Add a LGPL Notice and the GPLv3 and LGPLv3 licenses

    Requires Author and ProjectName variables.

    """
    src = 'licenses/lgpl'


class LicenseChoice(Skeleton):
    """Let the use pick the licence

    """
    variables = [
        Var('project_name'),
        Var('author'),
        Var('author_email'),
        Var('license', description='BSD/GPL/LGPL/other', default=''),
        ]

    supported_licenses = {
        'BSD' : BSD,
        'GPL' : GPL,
        'LGPL': LGPL
        }
    default_license = NoLicense
    _licence_skel = None

    @property
    def license_skel(self):
        """Return the skeleton for the set License.

        """
        if self._licence_skel is None:
            self._licence_skel = self.supported_licenses.get(
                self.get('license').upper(),
                self.default_license
                )(self)
        return self._licence_skel

    def check_variables(self):
        """Check variables of the license skeleton.

        """
        super(LicenseChoice, self).check_variables()
        self.license_skel.check_variables()

    def get_missing_variables(self):
        """Prompt for the license skeleton variables.

        """
        super(LicenseChoice, self).get_missing_variables()
        self.license_skel.get_missing_variables()

    def write(self, dst, run_dry=False):
        """Apply the license skeleton

        """
        self.license_skel.write(dst, run_dry=run_dry)



"""
Tests for skeleton core components

"""
import datetime
import unittest

from skeleton.tests.utils import TestCase, TempDir
from skeleton.core import Skeleton, Var, TemplateKeyError, FileNameKeyError, \
    Bool


THIS_YEAR = datetime.datetime.utcnow().year


class WithDefault(Skeleton):
    """Skeleton with variables with default"""
    variables = [Var('foo'), Var('bar', default=2), Var('baz')]


class WithRequirement(Skeleton):
    """Skeleton with other skeleton required"""
    variables = [Var('foo', default=1)]
    required_skeletons = [WithDefault, ]


class Static(Skeleton):
    """Skeleton with only static files:

    - foo.txt
    - bar/baz.txt

    """
    src = 'skeletons/static'


class DynamicContent(Static):
    """Skeleton dynamic content (bar/bax.txt_tmpl)"""
    src = 'skeletons/dynamic-content'
    variables = [
        Var('baz', 'Dummy variable'),
        ]


class DynamicFileName(Static):
    """Skeleton with a dynamic file name (bar/${baz}.txt)"""
    src = 'skeletons/dynamic-file-name'


class Required(Static):
    """Just a ${FileName}.txt file"""
    src = "skeletons/required"
    variables = [Var('file_name') ]


class StaticWithRequirement(Static):
    """Adds the requirment to the Static class"""
    required_skeletons = [Required]


class MissingVariable(DynamicContent):
    """We forgot to declare the variable "baz"
    """
    variables = []


class MissingVariableForFileName(DynamicFileName):
    """We forgot to declare the variable "baz"
    """
    variables = []


class TestSkeleton(TestCase):
    """Tests new implementation of Skeleton"""

    def test_skeleton_mapping(self):
        """Tests Skeleton is a Mapping class"""
        skel = Skeleton(foo=1)
        skel['bar'] = 2
        skel['baz'] = 3
        del skel['baz']

        self.assertTrue('foo' in skel)
        self.assertTrue('bar' in skel)
        self.assertFalse('baz' in skel)

        self.assertEqual(skel['foo'], 1)
        self.assertEqual(skel['bar'], 2)
        self.assertRaises(KeyError, skel.__getitem__, 'baz')

        self.assertEqual(skel.get('foo'), 1)
        self.assertEqual(skel.get('bar'), 2)
        self.assertEqual(skel.get('baz'), None)
        self.assertEqual(skel.get('baz', 3), 3)

        self.assertEqual(dict(skel), dict(foo=1, bar=2, year=THIS_YEAR))
        self.assertEqual(dict(**skel), dict(foo=1, bar=2, year=THIS_YEAR))

    def test_mapping_with_default(self):
        """Tests Skeleton correct map default value"""
        # foo is set, bar has a default, baz is not set and has no default
        skel = WithDefault(foo=1)

        self.assertTrue('foo' in skel)
        self.assertTrue('bar' in skel)
        self.assertFalse('baz' in skel)

        self.assertEqual(skel['foo'], 1)
        self.assertEqual(skel['bar'], 2)
        self.assertRaises(KeyError, skel.__getitem__, 'baz')

        self.assertEqual(skel.get('foo'), 1)
        self.assertEqual(skel.get('bar'), 2)
        self.assertEqual(skel.get('baz'), None)
        self.assertEqual(skel.get('baz', 3), 3)

        self.assertEqual(dict(skel), dict(foo=1, bar=2, year=THIS_YEAR))
        self.assertEqual(dict(**skel), dict(foo=1, bar=2, year=THIS_YEAR))

    def test_requirement_instances(self):
        """Tests skeleton and required skeletons hold the same set_variables
        """
        # For skel, foo is not set but has default set to 1
        skel = WithRequirement()

        # required_skel has foo set to 1 (get it from skel), bar is not set but
        # has a default set to 2; baz is not set and no default
        required_skel = skel.required_skeletons_instances[0]

        self.assertEqual(len(skel.required_skeletons_instances), 1)
        self.assertTrue(isinstance(required_skel, WithDefault))

        self.assertEqual(skel['foo'], 1)
        self.assertEqual(required_skel['foo'], 1)

        self.assertEqual(skel.get('bar'), None)
        self.assertEqual(required_skel['bar'], 2)

        # both skeleton should have baz set to 3
        required_skel['baz'] = 3

        self.assertEqual(skel['baz'], 3)
        self.assertEqual(required_skel['baz'], 3)

    def test_default_variables(self):
        """Tests Skeleton set the default Year variable."""
        skel = Skeleton()
        self.assertTrue('year' in skel)

    def test_check_var_with_default_var(self):
        """Tests Skeleton.check_va() on no set variables with defaults"""
        skel = WithDefault(foo=1, baz=3)
        try:
            skel.check_variables()
        except KeyError:
            self.fail("check_variables() should not raise KayError "
                "if the missing variable has a default.")

    def test_check_var_fails(self):
        """Tests Skeleton.check_var() on exception to raise."""
        skel = WithDefault(foo=1)
        self.assertRaises(KeyError, skel.check_variables)

    def test_get_variables_with_default(self):
        """Tests prompt of variable with default"""
        resps = ['', '1', '', '3']
        self.input_mock.side_effect = lambda x: resps.pop(0)

        skel = WithDefault()

        skel.get_missing_variables()
        self.assertEqual(self.input_mock.call_count, 4)
        self.assertEqual(skel['foo'], '1')
        self.assertEqual(skel['bar'], 2)
        self.assertEqual(skel['baz'], '3')

    def test_write_without_src(self):
        """tests skeleton src pointing to a missing folder"""
        skel = Skeleton()
        with TempDir() as tmp_dir:
            self.assertRaises(AttributeError, skel.write, tmp_dir.path)

    def test_write_missing_variable(self):
        """Tests write raise KeyError if a variable is not set."""
        skel = MissingVariable()
        with TempDir() as tmp_dir:
            try:
                skel.write(tmp_dir.path)
                self.fail("An exception should be raised")
            except (TemplateKeyError,), exc:
                self.assertTrue(exc.file_path.endswith('bar/baz.txt_tmpl'))
                self.assertEqual(exc.variable_name, 'baz')

    def test_write_create_dst_dir(self):
        """tests Skeleton.write() create the missing dst directory"""
        skel = Static()
        with TempDir() as tmp_dir:
            skel.write(tmp_dir.join('missing-dir'))
            self.assertEqual(
                open(tmp_dir.join('missing-dir/foo.txt')).read().strip(),
                'foo'
                )
            self.assertEqual(
                open(tmp_dir.join('missing-dir/bar/baz.txt')).read().strip(),
                'baz'
                )

    def test_write_static_file(self):
        """Tests Skeleton.write() with static file"""
        skel = Static()
        with TempDir() as tmp_dir:
            skel.write(tmp_dir.path)
            self.assertEqual(
                open(tmp_dir.join('foo.txt')).read().strip(),
                'foo'
                )
            self.assertEqual(
                open(tmp_dir.join('bar/baz.txt')).read().strip(),
                'baz'
                )

    def test_write_dynamic_content(self):
        """Tests Skeleton.write() with dynamic content."""
        skel = DynamicContent(baz="<replaced>")
        with TempDir() as tmp_dir:
            skel.write(tmp_dir.path)
            self.assertEqual(
                open(tmp_dir.join('foo.txt')).read().strip(),
                'foo'
                )
            self.assertEqual(
                open(tmp_dir.join('bar/baz.txt')).read().strip(),
                'foo <replaced> bar'
                )

    def test_write_dynamic_file_names(self):
        """Tests Skeleton.write() with dynamic file name"""
        skel = DynamicFileName(baz="replaced-name")
        with TempDir() as tmp_dir:
            skel.write(tmp_dir.path)
            self.assertEqual(
                open(tmp_dir.join('foo.txt')).read().strip(),
                'foo'
                )
            self.assertEqual(
                open(tmp_dir.join('bar/replaced-name.txt')).read().strip(),
                'baz'
                )

    def test_write_file_name_fails(self):
        """Tests Skeleton.write() with dynamic file name fails"""
        skel = MissingVariableForFileName()
        with TempDir() as tmp_dir:
            try:
                skel.write(tmp_dir.path)
                self.fail("An exception should be raised")
            except (FileNameKeyError,), exc:
                self.assertTrue(exc.file_path.endswith('bar/{baz}.txt'))
                self.assertEqual(exc.variable_name, 'baz')

    def test_run_with_var(self):
        """Tests Skeleton.run() with dynamic content and variable prompt."""
        resps = ['<input replacement>']
        self.input_mock.side_effect = lambda x: resps.pop(0)

        skel = DynamicContent()

        with TempDir() as tmp_dir:
            skel.run(tmp_dir.path)

            self.assertEqual(
                open(tmp_dir.join('foo.txt')).read().strip(),
                'foo'
                )
            self.assertEqual(
                open(tmp_dir.join('bar/baz.txt')).read().strip(),
                'foo <input replacement> bar'
                )

    def test_write_required_skel(self):
        """Tests it write the of required """
        skel = StaticWithRequirement(file_name="fooz")
        with TempDir() as tmp_dir:
            skel.write(tmp_dir.path)

            self.assertTrue(tmp_dir.exists('foo.txt'))
            self.assertTrue(tmp_dir.exists('bar/baz.txt'))
            self.assertTrue(tmp_dir.exists('fooz.txt'))

    def test_overwrite_required_skel(self):
        """Tests it write the of required """
        skel = StaticWithRequirement(file_name="foo")
        with TempDir() as tmp_dir:
            skel.write(tmp_dir.path)

            with open(tmp_dir.join('foo.txt')) as foo_file:
                self.assertEqual(foo_file.read().strip(), 'foo')


class TestVar(TestCase):
    """Tests for skeleton.Var"""

    def test_repr(self):
        """Tests Var representation"""
        var = Var('foo', description='dummy var')
        self.assertEqual(repr(var), '<Var foo default=None>')

    def test_full_description(self):
        """Tests Var full description (complete)"""
        var = Var('foo', description='dummy var')
        self.assertEqual(var.full_description, 'Foo (dummy var)')

    def test_basic_full_description(self):
        """Tests Var full description (missing description)"""
        var = Var('foo')
        self.assertEqual(var.full_description, 'Foo')

    def test_pep8_name(self):
        """Tests Var full description (missing description)"""
        var = Var('foo_bar')
        self.assertEqual(var.full_description, 'Foo Bar')

    def test_prompt(self):
        """Tests Var.prompt()"""
        var = Var('foo')
        self.assertEqual(var.prompt, 'Enter Foo: ')

    def test_do_prompt(self):
        """Tests Var.do_prompt()
        """
        resps = ['', 'bar']
        self.input_mock.side_effect = lambda x: resps.pop(0)

        var = Var('foo')
        self.assertEqual(var.do_prompt(), 'bar')
        self.assertEqual(self.input_mock.call_count, 2)

    def test_prompt_with_default(self):
        """Tests Var.prompt with default"""
        var = Var('foo', default='baz')
        self.assertEqual(var.prompt, """Enter Foo ['baz']: """)

    def test_do_prompt_with_default(self):
        """Tests Var.do_prompt() with default"""
        resps = ['']
        self.input_mock.side_effect = lambda x: resps.pop(0)

        var = Var('foo', default='baz')
        self.assertEqual(var.do_prompt(), 'baz')
        self.assertEqual(self.input_mock.call_count, 1)

    def test_prompt_empty_default(self):
        """Tests Var.prompt with empty default"""
        var = Var('foo', default='')
        self.assertEqual(var.prompt, """Enter Foo ['']: """)

    def test_do_prompt_empty_default(self):
        """Tests Var.do_prompt() with empty default"""
        resps = ['']
        self.input_mock.side_effect = lambda x: resps.pop(0)

        var = Var('foo', default='')
        self.assertEqual(var.do_prompt(), '')
        self.assertEqual(self.input_mock.call_count, 1)


class TestBool(TestCase):
    """Tests for skeleton.core.Bool"""

    def test_full_description(self):
        """Tests Bool full description (complete)"""
        var = Bool('foo', description='dummy var')
        self.assertEqual(var.full_description, 'Foo (dummy var - y/N)')

    def test_basic_full_description(self):
        """Tests Bool full description (complete)"""
        var = Bool('foo')
        self.assertEqual(var.full_description, 'Foo (y/N)')

    def test_prompt_true(self):
        """Tests Bool.prompt for True"""
        var = Bool('foo')
        self.assertEqual(var.prompt, 'Enter Foo (y/N): ')

    def test_do_prompt_true(self):
        """Tests Bool.do_prompt() for True"""
        resps = ['', 'y']
        self.input_mock.side_effect = lambda x: resps.pop(0)

        var = Bool('foo')
        self.assertEqual(var.do_prompt(), True)
        self.assertEqual(self.input_mock.call_count, 2)

    def test_do_prompt_false(self):
        """Tests Bool.do_prompt() for False"""
        resps = ['', 'n']
        self.input_mock.side_effect = lambda x: resps.pop(0)

        var = Bool('foo')
        self.assertEqual(var.do_prompt(), False)
        self.assertEqual(self.input_mock.call_count, 2)

    def test_prompt_with_default(self):
        """Tests Bool.prompt with default"""
        var = Bool('foo', default=True)
        self.assertEqual(var.prompt, "Enter Foo (y/N) ['y']: ")

    def test_do_prompt_with_default(self):
        """Tests Bool.do_prompt() with default"""
        resps = ['', ]
        self.input_mock.side_effect = lambda x: resps.pop(0)

        var = Bool('foo', default=False)
        self.assertEqual(var.do_prompt(), False)
        self.assertEqual(self.input_mock.call_count, 1)

    def test_do_prompt_default_overwritten(self):
        """Tests Bool.do_prompt() with default"""
        resps = ['y', ]
        self.input_mock.side_effect = lambda x: resps.pop(0)

        var = Bool('foo', default=False)
        self.assertEqual(var.do_prompt(), True)
        self.assertEqual(self.input_mock.call_count, 1)


class TestDefaultTemplate(unittest.TestCase):
    """Tests for the default template formatter"""

    def test_template_formatter(self):
        """Tests template formatting"""
        skel = Skeleton(bar="substituted")
        self.assertEqual(
            skel.template_formatter("""foo {bar} baz"""),
            """foo substituted baz""")

    def test_formatter_raise_key_error(self):
        """Tests template formatting a variable not set"""
        skel = Skeleton(bar="substituted")
        self.assertRaises(KeyError,
            skel.template_formatter, """foo {bar} {fooz} baz""")

    def test_template_use_default(self):
        """Tests the template uses the default value if the variable is not set
        """
        skel = WithDefault(foo=1, baz=3)
        self.assertEqual(
            skel.template_formatter("""{foo} {bar} {baz}"""),
            """1 2 3""")


def suite():
    """Get all licence releated test"""
    tests = unittest.TestSuite()
    tests.addTest(unittest.TestLoader().loadTestsFromTestCase(TestSkeleton))
    tests.addTest(unittest.TestLoader().loadTestsFromTestCase(TestVar))
    tests.addTest(unittest.TestLoader().loadTestsFromTestCase(TestBool))
    tests.addTest(
        unittest.TestLoader().loadTestsFromTestCase(TestDefaultTemplate))
    return tests

if __name__ == "__main__":
    unittest.main()



"""
Basic Template system for project skeleton,
similar to the template part of PasteScript but without any dependencies.

"""

from skeleton.core import (
    Skeleton, Var, Bool, FileNameKeyError, TemplateKeyError
    )
from skeleton.utils import insert_into_file
